"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const mixedContentSourceAdapter_1 = require("./mixedContentSourceAdapter");
const registry_1 = require("../../utils/registry");
const path_1 = require("path");
const utils_1 = require("../../utils");
/**
 * Handles decomposed types. A flavor of mixed content where a component can
 * have additional -meta.xml files that represent child components of the main
 * component.
 *
 * __Example Types__:
 *
 * CustomObject, CustomObjectTranslation
 *
 * __Example Structures__:
 *
 *```text
 * foos/
 * ├── MyFoo__c/
 * |   ├── MyFoo__c.foo-meta.xml
 * |   ├── bars/
 * |      ├── a.bar-meta.xml
 * |      ├── b.bar-meta.xml
 * |      ├── c.bar-meta.xml
 *```
 */
class DecomposedSourceAdapter extends mixedContentSourceAdapter_1.MixedContentSourceAdapter {
    constructor() {
        super(...arguments);
        this.ownFolder = true;
    }
    populate(component) {
        const parentPath = path_1.dirname(component.xml);
        component.children = this.getChildren(parentPath);
        return component;
    }
    getChildren(dirPath) {
        const children = [];
        for (const fileName of this.tree.readDirectory(dirPath)) {
            const currentPath = path_1.join(dirPath, fileName);
            if (this.forceIgnore.denies(currentPath)) {
                continue;
            }
            else if (this.tree.isDirectory(currentPath)) {
                children.push(...this.getChildren(currentPath));
            }
            else {
                const childXml = registry_1.parseMetadataXml(fileName);
                const fileIsRootXml = childXml.suffix === this.type.suffix;
                if (childXml && !fileIsRootXml) {
                    // TODO: Log warning if missing child type definition
                    const childTypeId = this.type.children.suffixes[childXml.suffix];
                    children.push({
                        fullName: utils_1.baseName(fileName),
                        type: this.type.children.types[childTypeId],
                        xml: currentPath
                    });
                }
            }
        }
        return children;
    }
}
exports.DecomposedSourceAdapter = DecomposedSourceAdapter;
//# sourceMappingURL=decomposedSourceAdapter.js.map